#include <errno.h>
#include <stdio.h>
#include <string.h>

#include "vpoppassd.h"
#include "vpp_err.h"

static char	cvs_id[] __unused =
"$Id: vpp_err.c,v 1.1 2001/08/08 17:35:42 roam Exp $";

static const char *errs[VPP_ERR_LAST] = {
	"no error",
	"out of memory",
	"bad command-line arguments",
	"unknown flavor",
	"no match for the specified flavor",
	"could not find a username in the environment",
	"pipe error",
	"forking child process",
	"program execution error",
	"waiting for child to terminate",
	"reading input",
	"writing output",
	"INTERNAL ERROR"
};

static int errsn[VPP_ERR_LAST] = {
	0, 0, 0,	/* NONE, NOMEM, CMDLINE */
	0, 0,		/* FL_UNKNOWN, FL_NOMATCH */
	0,		/* NOUSER */
	1, 1, 1, 1,	/* PIPE, FORK, EXEC, WAIT */
	1, 1,		/* READ, WRITE */
	0		/* INT */
};

static char	errbuf[512];

/*
 * Function:
 * 	vpp_strerror			- build an error message string
 * Inputs:
 * 	code				- error code
 * Returns:
 * 	pointer to static error message string
 * Modifies:
 * 	errbuf
 */

char *
vpp_strerror(vpp_err_t code) {
	
	if (code > VPP_ERR_INT)
		code = VPP_ERR_INT;

	if (errsn[code])
		snprintf(errbuf, sizeof(errbuf), "%s: %s", errs[code],
		    strerror(errno));
	else
		snprintf(errbuf, sizeof(errbuf), "%s", errs[code]);
	
	return (errbuf);
}

/*
 * Function:
 * 	vpp_prerror			- print error message and context
 * Inputs:
 * 	s				- context string to be printed
 * 	code				- error code
 * Returns:
 * 	code
 * Modifies:
 * 	nothing; writes to stderr
 */

vpp_err_t
vpp_prerror(const char *s, vpp_err_t code) {
	
	if (code > VPP_ERR_INT)
		code = VPP_ERR_INT;
	
	fprintf(stderr, "%s: %s\r\n", s, vpp_strerror(code));
	return (code);
}
