#include <sys/types.h>
#include <sys/wait.h>

#include <stdlib.h>
#include <string.h>
#include <unistd.h>

#include "vpoppassd.h"
#include "vpp_flavor.h"
#include "vpp_fl_bsdpw.h"
#include "vpp_util.h"

#ifdef EBUG
#include <stdio.h>
#endif

static char	cvs_id[] __unused =
"$Id: vpp_fl_bsdpw.c,v 1.3 2001/08/22 09:59:11 roam Exp $";

static const char	*bsdpw_prog_def[] = {
	BSDPW_PATH
};
#define BSDPW_PROG_DEF	(sizeof(bsdpw_prog_def)/sizeof(bsdpw_prog_def[0]))

static const char	*bsdpw_args_def[] = {
	"usermod",
	"%U",
	"-h0"
};
#define BSDPW_ARGS_DEF	(sizeof(bsdpw_args_def)/sizeof(bsdpw_args_def[0]))

static char		**bsdpw_prog;

static vpp_err_t	bsdpw_probe(int argc, const char *argv[]);
static vpp_err_t	bsdpw_init(int argc, const char *argv[]);
static vpp_err_t	bsdpw_change(const char *pass);

struct vpp_flavor_hnd	vpp_flh_bsdpw = {
	VPP_FLT_OUT,
	"bsdpw",
	VPP_FL_BSDPW,
	bsdpw_probe,
	bsdpw_init,
	bsdpw_change
};

static vpp_err_t
bsdpw_probe(int argc, const char *argv[]) {

	if (argc == 1)
		return (VPP_ERR_FL_NOMATCH);
	if (strcmp(argv[1], "pw"))
		return (VPP_ERR_FL_NOMATCH);

	return (VPP_ERR_NONE);
}

static vpp_err_t
bsdpw_init(int argc, const char *argv[]) {
	char **v;
	unsigned c, i;
	const char *str;
	vpp_err_t r;

	c = 0;
	v = NULL;

	if (r = vpp_argv_start(&c, &v), r)
			return (r);

	do {
		/* fake loop so I can break out */
		if (argc > 1) {
			if (r = vpp_argv_add(&c, &v, argc - 1, argv + 1), r)
				break;
		} else {
			if (str = getenv("VPP_BSDPW_PROG"), str != NULL) {
				/* break it down.. */
				return (VPP_ERR_INT);
			} else {
				if (r = vpp_argv_add(&c, &v,
				    BSDPW_PROG_DEF, bsdpw_prog_def), r)
					break;
			}
			if (str = getenv("VPP_BSDPW_ARGS"), str != NULL) {
				/* break it down.. */
				return (VPP_ERR_INT);
			} else {
				if (r = vpp_argv_add(&c, &v,
				    BSDPW_ARGS_DEF, bsdpw_args_def), r)
					break;
			}
		}

		for (i = 0; i < c; i++)
			RDBG(("v[%d] = %s\n", i, v[i]));
	} while (0);

	bsdpw_prog = v;
	return (VPP_ERR_NONE);
}

static vpp_err_t
bsdpw_change(const char *pass) {
	int pipes[2];
	char *output, *noutput, *line;
	size_t nsz, sz;
	unsigned i;
	pid_t pid;
	int status;
	vpp_err_t r;

	/* Spawn pw(8).. */
	if (r = vpp_argv_subst(bsdpw_prog), r)
		return (r);
	if (r = vpp_spawn(pipes, bsdpw_prog[0], bsdpw_prog), r)
		return (r);
	if (pid = waitpid(-1, &status, WNOHANG), pid == -1)
		return (VPP_ERR_WAIT);
	if (pid == 0) {
		if (r = vpp_writeline(pipes[1], 0, pass), r)
			return (r);
	}

	/* Gather output */
	output = NULL;
	sz = 0;
	while (r = vpp_readline(pipes[0], &line, &nsz), r == VPP_ERR_NONE) {
		if (line == NULL)
			break;

		for (i = 0; i < nsz; i++)
			if ((line[i] == '\r') || (line[i] == '\n'))
				line[i] = '.';
		if (noutput = realloc(output, sz + nsz), noutput == NULL) {
			free(output);
			return (VPP_ERR_NOMEM);
		}
		memcpy(noutput + sz, line, nsz);
		output = noutput;
		sz += nsz;
	}

	if (pid == 0) {
		if (pid = waitpid(pid, &status, 0), pid == -1) {
			free(output);
			return (VPP_ERR_WAIT);
		}
	}
	if (WIFEXITED(status) && (WEXITSTATUS(status) == 0))
		write(STDOUT_FILENO, "+OK ", 3);
	else
		write(STDOUT_FILENO, "-ERR ", 5);
	if (output != NULL)
		write(STDOUT_FILENO, output, sz);
	write(STDOUT_FILENO, "\r\n", 2);

	free(output);
	return (VPP_ERR_NONE);
}
