#include <sys/types.h>
#include <sys/wait.h>

#include <stdlib.h>
#include <string.h>
#include <unistd.h>

#include "vpoppassd.h"
#include "vpp_flavor.h"
#include "vpp_fl_vpasswd.h"
#include "vpp_util.h"

#ifdef EBUG
#include <stdio.h>
#endif

static char	cvs_id[] __unused =
"$Id: vpp_fl_vpasswd.c,v 1.2 2001/08/22 09:59:11 roam Exp $";

static const char	*vpasswd_prog_def[] = {
	VPASSWD_PATH
};
#define VPASSWD_PROG_DEF	(sizeof(vpasswd_prog_def)/sizeof(vpasswd_prog_def[0]))

static const char	*vpasswd_args_def[] = {
	"%U",
	"%P",
};
#define VPASSWD_ARGS_DEF	(sizeof(vpasswd_args_def)/sizeof(vpasswd_args_def[0]))

static char		**vpasswd_prog;

static vpp_err_t	vpasswd_probe(int argc, const char *argv[]);
static vpp_err_t	vpasswd_init(int argc, const char *argv[]);
static vpp_err_t	vpasswd_change(const char *pass);

struct vpp_flavor_hnd	vpp_flh_vpasswd = {
	VPP_FLT_OUT,
	"vpasswd",
	VPP_FL_VPASSWD,
	vpasswd_probe,
	vpasswd_init,
	vpasswd_change
};

static vpp_err_t
vpasswd_probe(int argc, const char *argv[]) {

	if (argc == 1)
		return (VPP_ERR_FL_NOMATCH);
	if (strcmp(argv[1], "pw"))
		return (VPP_ERR_FL_NOMATCH);

	return (VPP_ERR_NONE);
}

static vpp_err_t
vpasswd_init(int argc, const char *argv[]) {
	char **v;
	unsigned c, i;
	const char *str;
	vpp_err_t r;

	c = 0;
	v = NULL;

	if (r = vpp_argv_start(&c, &v), r)
			return (r);

	do {
		/* fake loop so I can break out */
		if (argc > 1) {
			if (r = vpp_argv_add(&c, &v, argc - 1, argv + 1), r)
				break;
		} else {
			if (str = getenv("VPP_VPASSWD_PROG"), str != NULL) {
				/* break it down.. */
				return (VPP_ERR_INT);
			} else {
				if (r = vpp_argv_add(&c, &v,
				    VPASSWD_PROG_DEF, vpasswd_prog_def), r)
					break;
			}
			if (str = getenv("VPP_VPASSWD_ARGS"), str != NULL) {
				/* break it down.. */
				return (VPP_ERR_INT);
			} else {
				if (r = vpp_argv_add(&c, &v,
				    VPASSWD_ARGS_DEF, vpasswd_args_def), r)
					break;
			}
		}

		for (i = 0; i < c; i++)
			RDBG(("v[%d] = %s\n", i, v[i]));
	} while (0);

	vpasswd_prog = v;
	return (VPP_ERR_NONE);
}

static vpp_err_t
vpasswd_change(const char *pass __unused) {
	int pipes[2];
	char *output, *noutput, *line;
	size_t nsz, sz;
	unsigned i;
	pid_t pid;
	int status;
	vpp_err_t r;

	/* Spawn pw(8).. */
	if (r = vpp_argv_subst(vpasswd_prog), r)
		return (r);
	if (r = vpp_spawn(pipes, vpasswd_prog[0], vpasswd_prog), r)
		return (r);
	if (pid = waitpid(-1, &status, WNOHANG), pid == -1)
		return (VPP_ERR_WAIT);
	RDBG(("waitpid: pid=%d, status=%X\n", pid, status));
	/* Nothing to write. */

	/* Gather output */
	output = NULL;
	sz = 0;
	RDBG(("gathering output..\n"));
	while (r = vpp_readline(pipes[0], &line, &nsz), r == VPP_ERR_NONE) {
		if (line == NULL)
			break;

		for (i = 0; i < nsz; i++)
			if ((line[i] == '\r') || (line[i] == '\n'))
				line[i] = '.';
		RDBG(("Read %.*s\n", (int) sz, line));
		if (noutput = realloc(output, sz + nsz), noutput == NULL) {
			free(output);
			return (VPP_ERR_NOMEM);
		}
		memcpy(noutput + sz, line, nsz);
		output = noutput;
		sz += nsz;
	}
	RDBG(("finished gathering output, output=%p (%.*s), len=%d\n",
	    output, (int) sz, output, sz));

	if (pid == 0) {
		if (pid = waitpid(pid, &status, 0), pid == -1) {
			RDBG(("waitpid2: pid=%d, status=%X\n", pid, status));
			free(output);
			return (VPP_ERR_WAIT);
		}
	}
	RDBG(("test: status=%X, WIFEXITED=%d, WEXITSTATUS=%X\n",
	    status, WIFEXITED(status), WEXITSTATUS(status)));
	if (WIFEXITED(status) && (WEXITSTATUS(status) == 0))
		write(STDOUT_FILENO, "+OK ", 3);
	else
		write(STDOUT_FILENO, "-ERR ", 5);
	if (output != NULL)
		write(STDOUT_FILENO, output, sz);
	write(STDOUT_FILENO, "\r\n", 2);

	free(output);
	return (VPP_ERR_NONE);
}
