#include <stdlib.h>
#include <string.h>

#include "vpoppassd.h"
#include "vpp_flavor.h"

#include "vpp_fl_bsdpw.h"
#include "vpp_fl_checkpwd.h"
#include "vpp_fl_vchkpw.h"
#include "vpp_fl_vpasswd.h"

#ifdef EBUG
#include <stdio.h> /* FIXME: debug only? */
#endif

static char	cvs_id[] __unused =
"$Id: vpp_flavor.c,v 1.1 2001/08/08 17:35:42 roam Exp $";

static struct vpp_flavor_hnd	*flav[] = {
	/* input */
	&vpp_flh_vchkpw,
	&vpp_flh_checkpwd,
	/* output */
	&vpp_flh_bsdpw,
	&vpp_flh_vpasswd,
	NULL
};

static struct vpp_flavor_hnd	*cflavor[VPP_FLT_LAST];

static vpp_err_t	vpp_flavor_set_byhnd(struct vpp_flavor_hnd *h,
		int argc, const char *argv[]);

/*
 * Function:
 * 	vpp_flavor_set			- set a specified flavor
 * Inputs:
 * 	flavor				- the flavor to set
 * Returns:
 * 	vpp_err_t
 * 	nothing so far (FIXME)
 * Modifies:
 * 	nothing so far (FIXME)
 */

vpp_err_t
vpp_flavor_set(vpp_flavor_type t, const char *flavor, int argc, const char *argv[]) {
	unsigned i;

	for (i = 0; flav[i] != NULL; i++) {
		if (flav[i]->name == NULL)
			return (VPP_ERR_INT);
		if ((flav[i]->type == t) && !strcmp(flav[i]->name, flavor))
			break;
	}
	if (flav[i] == NULL)
		return (VPP_ERR_FL_UNKNOWN);

	return (vpp_flavor_set_byhnd(flav[i], argc, argv));
}

/*
 * Function:
 * 	vpp_flavor_setdefault		- set a default flavor based on args
 * Inputs:
 * 	argc, argv			- main() args
 * Returns:
 * 	vpp_err_t
 * 	FL_UNKNOWN			- could not determine the flavor
 * 	error code from the various flavor probe routines
 * Modifies:
 * 	sets cflavor to the detected flavor type
 * 	calls the various flavor probe routines
 */

vpp_err_t
vpp_flavor_probe(vpp_flavor_type t, int argc, const char *argv[]) {
	unsigned i;
	vpp_err_t r;

	for (i = 0; flav[i] != NULL; i++)
		if ((flav[i]->type == t) && (flav[i]->probe != NULL)) {
			r = flav[i]->probe(argc, argv);
			if (r == VPP_ERR_NONE)
				break;
			else if (r != VPP_ERR_FL_NOMATCH)
				return (r);
		}
	if (flav[i] == NULL)
		return (VPP_ERR_FL_UNKNOWN);

	return (vpp_flavor_set_byhnd(flav[i], argc, argv));
}

static vpp_err_t
vpp_flavor_set_byhnd(struct vpp_flavor_hnd *h, int argc, const char *argv[]) {
	const char *val;
	vpp_err_t r;

	if (h->init == NULL)
		return (VPP_ERR_INT);
	if (r = h->init(argc, argv), r)
		return (r);

	/* Do whatever else is necessary.. */
	if (h->type == VPP_FLT_IN) {
		/* grab the username */
		if (vpp_uservar == NULL)
			return (VPP_ERR_INT);
		if (val = getenv(vpp_uservar), val == NULL)
			return (VPP_ERR_NOUSER);
		vpp_user = val;
	} else if (h->type == VPP_FLT_OUT) {
		/* nothing to do here.. I think. */
	}
	
	cflavor[h->type] = h;
	return (VPP_ERR_NONE);
}

vpp_err_t
vpp_flavor_change(const char *pass) {

	/* cflavor should have been set by vpp_init() long ago.. */
	if ((cflavor[VPP_FLT_OUT] == NULL) ||
	    (cflavor[VPP_FLT_OUT]->change == NULL))
		return (VPP_ERR_INT);
	
	return (cflavor[VPP_FLT_OUT]->change(pass));
}
